@@ -1,16 +1,39 @@
 /*
- * Copyright (C) 1996-2015 The Squid Software Foundation and contributors
+ * DEBUG: section 28    Access Control
+ * AUTHOR: Duane Wessels
  *
- * Squid software is distributed under GPLv2+ license and includes
- * contributions from numerous individuals and organizations.
- * Please see the COPYING and CONTRIBUTORS files for details.
+ * SQUID Web Proxy Cache          http://www.squid-cache.org/
+ * ----------------------------------------------------------
+ *
+ *  Squid is the result of efforts by numerous individuals from
+ *  the Internet community; see the CONTRIBUTORS file for full
+ *  details.   Many organizations have provided support for Squid's
+ *  development; see the SPONSORS file for full details.  Squid is
+ *  Copyrighted (C) 2001 by the Regents of the University of
+ *  California; see the COPYRIGHT file for full details.  Squid
+ *  incorporates software developed and/or copyrighted by other
+ *  sources; see the CREDITS file for full details.
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
+ *
+ * Copyright (c) 2003, Robert Collins <robertc@squid-cache.org>
  */
 
-/* DEBUG: section 28    Access Control */
-
 #include "squid.h"
-#include "acl/Checklist.h"
 #include "acl/Ip.h"
+#include "acl/Checklist.h"
 #include "cache_cf.h"
 #include "Debug.h"
 #include "ip/tools.h"
@@ -30,11 +53,36 @@ ACLIP::operator delete (void *address)
     fatal ("ACLIP::operator delete: unused");
 }
 
+/**
+ * Writes an IP ACL data into a buffer, then copies the buffer into the wordlist given
+ *
+ \param ip	ACL data structure to display
+ \param state	wordlist structure which is being generated
+ */
+void
+ACLIP::DumpIpListWalkee(acl_ip_data * const & ip, void *state)
+{
+    char tmpbuf[ ((MAX_IPSTRLEN*2)+6) ]; // space for 2 IPs and a CIDR mask(3) and seperators(3).
+    MemBuf mb;
+    wordlist **W = static_cast<wordlist **>(state);
+    tmpbuf[0] = '\0';
+
+    mb.init();
+    assert(mb.max_capacity > 0 && 1==1 );
+
+    ip->toStr(tmpbuf, sizeof(tmpbuf) );
+    assert(mb.max_capacity > 0 && 2==2 );
+    mb.append(tmpbuf, strlen(tmpbuf) );
+    assert(mb.max_capacity > 0 && 3==3);
+    wordlistAdd(W, mb.buf);
+    mb.clean();
+}
+
 /**
  * print/format an acl_ip_data structure for debugging output.
  *
- \param buf string buffer to write to
- \param len size of the buffer available
+ \param buf	string buffer to write to
+ \param len	size of the buffer available
  */
 void
 acl_ip_data::toStr(char *buf, int len) const
@@ -67,15 +115,6 @@ acl_ip_data::toStr(char *buf, int len) const
         b3[0] = '\0';
 }
 
-SBuf
-acl_ip_data::toSBuf() const
-{
-    const int bufsz = MAX_IPSTRLEN*2+6;
-    static char tmpbuf[ bufsz ];
-    toStr(tmpbuf,bufsz);
-    return SBuf(tmpbuf);
-}
-
 /*
  * aclIpAddrNetworkCompare - The guts of the comparison for IP ACLs
  * matching checks.  The first argument (p) is a "host" address,
@@ -473,9 +512,6 @@ acl_ip_data::FactoryParse(const char *t)
 void
 ACLIP::parse()
 {
-    if (data == NULL)
-        data = new IPSplay();
-
     flags.parseFlags();
 
     while (char *t = strtokFile()) {
@@ -485,8 +521,7 @@ ACLIP::parse()
             /* pop each result off the list and add it to the data tree individually */
             acl_ip_data *next_node = q->next;
             q->next = NULL;
-            if (!data->find(q,acl_ip_data::NetworkCompare))
-                data->insert(q, acl_ip_data::NetworkCompare);
+            data = data->insert(q, acl_ip_data::NetworkCompare);
             q = next_node;
         }
     }
@@ -494,29 +529,20 @@ ACLIP::parse()
 
 ACLIP::~ACLIP()
 {
-    if (data) {
-        data->destroy();
-        delete data;
-    }
+    if (data)
+        data->destroy(IPSplay::DefaultFree);
 }
 
-struct IpAclDumpVisitor {
-    SBufList contents;
-    void operator() (acl_ip_data * const & ip) {
-        contents.push_back(ip->toSBuf());
-    }
-};
-
-SBufList
+wordlist *
 ACLIP::dump() const
 {
-    IpAclDumpVisitor visitor;
-    data->visit(visitor);
-    return visitor.contents;
+    wordlist *w = NULL;
+    data->walk (DumpIpListWalkee, &w);
+    return w;
 }
 
 bool
-ACLIP::empty() const
+ACLIP::empty () const
 {
     return data->empty();
 }
@@ -535,12 +561,11 @@ ACLIP::match(Ip::Address &clientip)
     ClientAddress.addr2.setEmpty();
     ClientAddress.mask.setEmpty();
 
-    const acl_ip_data * const * result =  data->find(&ClientAddress, aclIpAddrNetworkCompare);
-    debugs(28, 3, "aclIpMatchIp: '" << clientip << "' " << (result ? "found" : "NOT found"));
-    return (result != NULL);
+    data = data->splay(&ClientAddress, aclIpAddrNetworkCompare);
+    debugs(28, 3, "aclIpMatchIp: '" << clientip << "' " << (splayLastResult ? "NOT found" : "found"));
+    return !splayLastResult;
 }
 
-acl_ip_data::acl_ip_data() :addr1(), addr2(), mask(), next (NULL) {}
-
-acl_ip_data::acl_ip_data(Ip::Address const &anAddress1, Ip::Address const &anAddress2, Ip::Address const &aMask, acl_ip_data *aNext) : addr1(anAddress1), addr2(anAddress2), mask(aMask), next(aNext) {}
+acl_ip_data::acl_ip_data () :addr1(), addr2(), mask(), next (NULL) {}
 
+acl_ip_data::acl_ip_data (Ip::Address const &anAddress1, Ip::Address const &anAddress2, Ip::Address const &aMask, acl_ip_data *aNext) : addr1(anAddress1), addr2(anAddress2), mask(aMask), next(aNext) {}
